// Microsoft ATP Safe Links Cleaner
// Copyright 2021 David Byers <david.byers@liu.se>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

// Shared code


const safelinksRegexp = new RegExp(
    'https?://[^.]+[.]safelinks[.]protection[.]outlook[.]com/[?]url=([^&]+)&.*',
    'gi'
);

function untangleLink(link) {
    return link.replaceAll(
	safelinksRegexp, (match, url) => {
	    try {
		return decodeURIComponent(url);
	    }
	    catch (e) {
		console.log(e);
		return url;
	    }
	});
}

function isTangledLink(link) {
    return link.match(safelinksRegexp);
}

function getTextNodes(elem) {
    var result = [];
    if (elem) {
	for (var nodes = elem.childNodes, i = nodes.length; i--;) {
	    let node = nodes[i];
	    let nodeType = node.nodeType;
	    
	    if (nodeType == Node.TEXT_NODE) {
		result.push(node);
	    }
	    else if (nodeType == Node.ELEMENT_NODE
		     || nodeType == Node.DOCUMENT_NODE
		     || nodeType == Node.DOCUMENT_FRAGMENT_NODE) {
		result = result.concat(getTextNodes(node));
	    }
	}
    }
    return result;
}
